package kr.co.vividnext.sodalive.live

import io.reactivex.rxjava3.core.Flowable
import io.reactivex.rxjava3.core.Single
import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.live.reservation.MakeLiveReservationRequest
import kr.co.vividnext.sodalive.live.room.CancelLiveRequest
import kr.co.vividnext.sodalive.live.room.EnterOrQuitLiveRoomRequest
import kr.co.vividnext.sodalive.live.room.LiveRoomStatus
import kr.co.vividnext.sodalive.live.room.StartLiveRequest
import kr.co.vividnext.sodalive.live.room.create.CreateLiveRoomResponse
import kr.co.vividnext.sodalive.live.room.detail.GetRoomDetailResponse
import okhttp3.MultipartBody
import okhttp3.RequestBody
import java.util.TimeZone

class LiveRepository(private val api: LiveApi) {
    fun roomList(
        dateString: String? = null,
        status: LiveRoomStatus,
        page: Int,
        size: Int,
        token: String
    ): Flowable<ApiResponse<List<GetRoomListResponse>>> {
        return api.roomList(
            timezone = TimeZone.getDefault().id,
            dateString = dateString,
            status = status,
            page = page - 1,
            size = size,
            authHeader = token
        )
    }

    fun getRoomDetail(roomId: Long, token: String): Single<ApiResponse<GetRoomDetailResponse>> {
        return api.getRoomDetail(
            roomId,
            timezone = TimeZone.getDefault().id,
            authHeader = token
        )
    }

    fun getRecentRoomInfo(token: String) = api.getRecentRoomInfo(authHeader = token)

    fun createRoom(
        coverImage: MultipartBody.Part? = null,
        request: RequestBody,
        token: String
    ): Single<ApiResponse<CreateLiveRoomResponse>> {
        return api.createRoom(
            coverImage,
            request,
            authHeader = token
        )
    }

    fun startLive(
        request: StartLiveRequest,
        token: String
    ) = api.startLive(
        request,
        authHeader = token
    )

    fun cancelLive(
        request: CancelLiveRequest,
        token: String
    ) = api.cancelLive(request, authHeader = token)

    fun enterRoom(
        request: EnterOrQuitLiveRoomRequest,
        token: String
    ) = api.enterRoom(
        request,
        authHeader = token
    )

    fun makeReservation(
        request: MakeLiveReservationRequest,
        token: String
    ) = api.makeReservation(
        request,
        authHeader = token
    )

    fun editLiveRoomInfo(
        roomId: Long,
        coverImage: MultipartBody.Part? = null,
        request: RequestBody? = null,
        token: String
    ) = api.editLiveRoomInfo(
        id = roomId,
        coverImage = coverImage,
        request = request,
        authHeader = token
    )
}
